@@ -12,6 +12,8 @@ import dill
 import json
 import threading
 from tornado.ioloop import IOLoop
+from urllib.parse import quote
+
 
 try:
     import builtins
@@ -46,6 +48,8 @@ __all__ = [
     'get_instances_by_class',
     'to_json',
     'cm', 'call_method',
+    'sw', 'sliding_window',
+    'read_from',
 ]
 
 
@@ -104,19 +108,19 @@ class Pipe:
 
     def __rrshift__(self, other):
         """左边的 >>."""
-        # 左边如果支持sink方法，则变为左边sin右边的函数
+        # 左边如果支持sink方法，则变为左边sink右边的函数
         if hasattr(other, 'sink'):
             return other.sink(self.func)
         else:
-            return self.func(other)
+            return self.__ror__(other)
 
     def __rmatmul__(self, other):
         """左边的 @."""
-        return self.func(other)
+        return self.__ror__(other)
 
     def __lshift__(self, other):  # 右边的<<
         """右边的 <<."""
-        return self.func(other)
+        return self.__ror__(other)
 
     def __call__(self, *args, **kwargs):
         """像正常函数一样使用使用."""
@@ -640,8 +644,8 @@ def size(x):
     return sys.getsizeof(x)
 
 
-@Pipe
-def post_to(url='http://127.0.0.1:7777', asynchronous=True, headers={}):
+@P
+def post_to(url='http://127.0.0.1:7777', tag='', asynchronous=True, headers={}):
     """ post a str or bytes or pyobject to url.
 
     str:直接发送
@@ -656,16 +660,16 @@ def post_to(url='http://127.0.0.1:7777', asynchronous=True, headers={}):
         {'a':1}>>post_to(url,asynchronous=False)
 
     """
+    headers.update({'tag': quote(tag)})
+
     def _encode(body):
-        if not isinstance(body, bytes):
-            try:
-                body = json.dumps(body, ensure_ascii=False)
-            except TypeError:
-                body = dill.dumps(body)
+        body = dill.dumps(body)
         return body
 
     @gen.coroutine
     def _async(body):
+        if not isinstance(body, list):
+            body = [body]
         body = _encode(body)
         from tornado import httpclient
         http_client = httpclient.AsyncHTTPClient()
@@ -685,6 +689,72 @@ def post_to(url='http://127.0.0.1:7777', asynchronous=True, headers={}):
         return _sync @ P
 
 
+@Pipe
+def read_from(url='http://127.0.0.1:7777', tag='', asynchronous=True, headers={}):
+    """ post a str or bytes or pyobject to url.
+
+    str:直接发送
+    bytes:直接发送
+    pyobject:dill序列化后发送
+    发送方式use async http client,Future对象，jupyter中可直接使用
+    jupyter 之外需要loop = IOLoop.current(instance=True)，loop.start()
+
+    Examples::
+
+        {'a':1}>>post_to(url)
+        {'a':1}>>post_to(url,asynchronous=False)
+
+    """
+    headers.update({'User-Agent': 'deva'})
+    if tag:
+        url += quote(tag)
+
+    def _encode(body):
+        if not isinstance(body, bytes):
+            try:
+                body = json.dumps(body, ensure_ascii=False)
+            except TypeError:
+                body = dill.dumps(body)
+        return body
+
+    def _loads(body):
+        """解析从web端口提交过来的数据.
+
+        可能的数据有字符串和二进制,
+        字符串可能是直接字符串,也可能是json编码后的字符串
+        二进制可能是图像等直接可用的二进制,也可能是dill编码的二进制pyobject
+        """
+        try:
+            body = json.loads(body)
+        except TypeError:
+            body = dill.loads(body)
+        except ValueError:
+            body = body.decode('utf-8')
+        finally:
+            return body
+
+    @gen.coroutine
+    def _async():
+        from tornado import httpclient
+        http_client = httpclient.AsyncHTTPClient()
+        request = httpclient.HTTPRequest(
+            url, method="GET", headers=headers)
+        response = yield http_client.fetch(request)
+        body = dill.loads(response.body)
+        return body
+
+    def _sync():
+        import requests
+        body = requests.get(url, headers=headers)
+        body = _loads(body)
+        return body
+
+    if asynchronous:
+        return _async()
+    else:
+        return _sync()
+
+
 @Pipe
 def sample(samplesize=5):
     """从序列中取出随机的n个数据
@@ -849,7 +919,7 @@ def to_json(r):
 
 
 @Pipe
-def cm(mkey):
+def call_method(mkey):
     "call method by a method key"
 
     def _cm(obj):
@@ -864,9 +934,35 @@ def cm(mkey):
         return _cm(obj)
 
 
-call_method = cm
+cm = call_method
 
 
+@Pipe
+def sliding_window(qte: int = 2):
+    "Returns a sliding window (of width n) over data from the iterable"
+    "   s -> (s0,s1,...s[n-1]), (s1,s2,...,sn), ...     "
+    "range(100)>>sample(10)>>sw>>tee>>pmap(sum)>>tee>>max"
+    def _window(iterable):
+        i = qte
+        "Returns a sliding window (of width n) over data from the iterable"
+        "   s -> (s0,s1,...s[n-1]), (s1,s2,...,sn), ...                   "
+        it = iter(iterable)
+        result = tuple(islice(it, i))
+        if len(result) == i:
+            yield result
+        for elem in it:
+            result = result[1:] + (elem,)
+            yield result
+
+    if isinstance(qte, int):
+        return _window @ P
+    else:
+        iterable, qte = qte, 2
+        return _window(iterable)
+
+
+sw = sliding_window
+
 if __name__ == "__main__":
     import doctest
     doctest.testfile('README.md')
